Module: FatCore

Defined in:
lib/fat_core.rb,
lib/fat_core/nil.rb,
lib/fat_core/hash.rb,
lib/fat_core/array.rb,
lib/fat_core/range.rb,
lib/fat_core/string.rb,
lib/fat_core/symbol.rb,
lib/fat_core/numeric.rb,
lib/fat_core/version.rb,
lib/fat_core/bigdecimal.rb

Overview

Gem Overview (extracted from README.org by gem_docs)

  • Introduction ~fat-core~ is somewhat of a grab bag of core class extensions that I have found useful across several projects. It's higgeldy-piggeldy nature reflects the fact that none of them are important enough to deserve a gem of their own, but nonetheless need to be collected in one place to reduce redundancy across projects and provide a focused place to develop and test them.

Defined Under Namespace

Modules: Array, BigDecimal, Hash, NilClass, Numeric, Range, String, Symbol

Constant Summary collapse

MAJOR =
7
MINOR =
2
PATCH =
0
VERSION =

FatCore version number

[MAJOR, MINOR, PATCH].compact.join('.')